/*
 *  Copyright 2002-2009 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of BMTron.
 *
 *  BMTron is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  BMTron is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BMTron.  If not, see <http://www.gnu.org/licenses/>.
 */
 
package kryshen.bmtron;

import java.awt.Container;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

public class TextField extends Container 
        implements MouseListener, KeyListener, FocusListener {
    private final Game game;
    private boolean active = false;
    private final java.awt.TextField awtText;
    
    private String note = null;

    private final int width = 333;
    private final int height = 35;

    TextField(String text, Game bmt) {
	this.game = bmt;

        setLayout(null);
    
        awtText = new java.awt.TextField(text);
        awtText.setSize(width, height);
        awtText.setLocation(0, 0);	
        awtText.setVisible(false);
	awtText.setBackground(bmt.skin.buttonBackColor);
	awtText.setForeground(bmt.skin.buttonTextColor);
	awtText.setFont(bmt.skin.buttonFont);
        add(awtText);
        
        awtText.addKeyListener(this);
        awtText.addFocusListener(this);

        setSize(width, height);
        addMouseListener(this);
    }
    
    public void setText(String text) {
        awtText.setText(text);
        repaint();
    }
    
    public String getText() {
        return awtText.getText();
    }
    
    public void setNote(String note) {
        this.note = note;
        awtText.setVisible(false);
        repaint();
    }

    public String getNote() {
        return note;
    }

    public void paint(Graphics g) {
        g.setColor(game.skin.buttonBackColor);
        g.fillRect(0, 0, width - 1, height - 1);
        
        if (active) 
	    g.setColor(game.skin.activeButtonBorderColor); 
	else 
	    g.setColor(game.skin.buttonBorderColor);

        g.drawRect(0, 0, width - 1, height - 1);

	g.setColor(game.skin.buttonTextColor);

        String s = awtText.getText();
        if (note != null)
            s += ' ' + note;

        g.setFont(game.skin.buttonFont);
        FontMetrics fm = g.getFontMetrics();
        
        int sw = fm.stringWidth(s);
        int sh = fm.getHeight();        
        g.drawString(s, (width - sw) / 2, (height - sh) / 2 + fm.getAscent());
        
        super.paint(g);
    }
    
    public void mouseEntered(MouseEvent e) { active = true; repaint(); }
    public void mouseExited(MouseEvent e) { active = false; repaint(); }
    
    public void mousePressed(MouseEvent e) {
        awtText.setVisible(true);
        awtText.requestFocus();
    }
        
    public void mouseReleased(MouseEvent e) {}    
    public void mouseClicked(MouseEvent e) {}
    
    public void keyPressed(KeyEvent e) {
        //System.err.println(e.getKeyCode());
        if (e.getKeyCode() == 10) {
            awtText.setVisible(false);
	    requestFocus();
	}
    }
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}

    public void focusGained(FocusEvent e) {}
    
    public void focusLost(FocusEvent e) {
        awtText.setVisible(false);
    }
}
